summarize.results <- function(res,beta_true,digits=2){
  #PURPOSE: evaluate the criteria (bias, CI width, coverage, rmse) to assess the imputation methods
  
  #OUTPUT: the several criteria
  
  #INPUTS:
  #res: an output of the function Simulate.real
  #beta_true: the true value of the parameters
  #digits: argument for rounding
  
  bias_beta <- apply(sweep(res[,,,1],MARGIN=c(3),beta_true,FUN="-"),c(1,3),mean)
  isin_beta <- apply(res[,,,2],c(1,2),FUN=function(x,beta_true){x< beta_true},beta_true=beta_true) & 
    apply(res[,,,3],c(1,2),FUN=function(x,beta_true){beta_true < x},beta_true=beta_true)
  cov_beta<-t(apply(isin_beta,c(1,2),mean))
  
  intwidth_beta <- res[,,,3] - res[,,,2]
  aiw_beta<-apply(intwidth_beta,c(1,3),median)
  
  rmse<-sqrt(na.omit(apply(sweep(res[,,,1],MARGIN=c(3),beta_true,FUN="-")^2,c(1,3),mean)))
  results<-lapply(list(biais=na.omit(bias_beta),aiw=na.omit(aiw_beta),
                       cov=na.omit(cov_beta),rmse=rmse),round,digits=digits)
  return(results)
}

plot.summarize<-function(res.sum,beta_true=NULL,cov.lim=c(0.6,1),biais.lim=c(-15,15),nbsim,Name="Jeu",plot.cov=F,plot.ci=F,plot.bias=F){
  #PURPOSE: plot the criteria of the simulation
  
  #INPUTS:
  #res.sum: an output of the function Simulate.real
  #beta_true: the true value of the parameters
  #cov.lim, biais.lim: graphical parameters
  #nbsim: number of simulations (useful for the dashed line)
  #plot.cov,plot.ci,plot.bias: booleans to precise which criteria have to be ploted
  par(las=2,cex=1.5,mfrow=c(plot.cov+plot.ci+plot.bias,1))
  if(plot.cov){
    boxplot(t(res.sum$cov),main=Name,ylim=cov.lim,ylab="coverage")
    abline(0.95,0,lty=1,col="grey");
    abline(binom.confint(ceiling(0.95*nbsim), nbsim, conf.level = 0.95, methods = "agresti-coull")$lower,0,lty=2)
  }
  
  #CI
  if(plot.ci){
    if("LD"%in%rownames(res.sum$aiw)){
      boxplot(t(sweep(
        res.sum$aiw[-which(rownames(res.sum$aiw)=="LD"),],2,FUN="/",
        STATS=res.sum$aiw["LD",])),main=Name,ylab="increase of CI width",ylim=c(0.5,1.3))
      abline(1,0,col="grey",lty=1)
    }
  }
  
  #relative bias
  if(plot.bias){
    boxplot(t(sweep(
      res.sum$biais,2,FUN="/",
      STATS=beta_true)),main=Name,ylab="relative bias",ylim=biais.lim)
    abline(0,0,col="grey",lty=1)
  }
}


createdata.real<-function(X,Y,n.ech,formula.multinom){
  # PURPOSE: draw a sample from a population data set (X,Y)
  # and build the response variable according to a logistic regression model
  
  # OUTPUT: a (complete) sample drawn from the population
  
  # INPUTS:
  # X: covariates
  # Y: response variable
  # n.ech: size of the sample
  # formula.multinom: regression formula that explains the response
  X.ech<-X[sample(1:nrow(X),size=n.ech,replace=T),]
  model.true<-multinom(as.formula(formula.multinom),data=cbind.data.frame(X,Y),trace=F)
  beta_true<-matrix(t(coef(model.true)),length(coef(model.true)),1,byrow=T)
  proba.Y<-predict(model.true,newdata=X.ech,type="probs")
  if(is.matrix(proba.Y)){#regression multinomiale
    Y.ech<-apply(proba.Y,1,FUN=function(prob,x){sample(x=x,size=1,replace=F,prob=prob)},x=levels(Y[,1]))
    Y.ech<-factor(Y.ech,levels = levels(Y[,1]))
  }else if (is.vector(proba.Y)){#regression logistique
    proba.Y<-matrix(proba.Y,length(proba.Y),1)
    Y.ech<-apply(proba.Y,1,FUN=function(prob,x){sample(x=x,size=1,replace=F,prob=c(1-prob,prob))},x=levels(Y[,1]))
    Y.ech<-factor(Y.ech,levels = levels(Y[,1]))
  }
  
  res<-list(X=cbind.data.frame(X.ech,Y.ech),beta_true=beta_true)
  colnames(res$X)<-c(colnames(X),colnames(Y)[1])
  return(res)
}

LD.real<-function(data,formula.multinom,var.interest=NULL){
  # PURPOSE: calculate the estimates and the criteria using listwise deletion
  
  # OUTPUT: the estimates and the confidence interval for the quantities of interest
  
  # INPUTS:
  # data: an incomplete data set
  #var.interest: index of the variables used in the analysis model
  # formula.multinom: regression formula that explains the response
  if(!is.character(formula.multinom)){warning("formula.multinom must be a string")}
  if(is.null(var.interest)){var.interest<- c(1:ncol(data))}
  imp<-na.omit(data[,var.interest])
  res.mult<-multinom(formula=as.formula(formula.multinom),data=imp,trace=F)  
  CI<-confint(res.mult)
  if(is.matrix(CI)){
    res<-cbind(coef(res.mult),CI)
  }else{
    res<-matrix(NA,length(coef(res.mult)),3)
    res[,1]<-matrix(t(coef(res.mult)),nrow(res),1,byrow=T)
    res[,2]<-CI[,1,]
    res[,3]<-CI[,2,]
  }
  colnames(res)<-c("est","lo 95","hi 95")   
  return(res)
}


test.impute.real <- function(data, method, formula.multinom=NULL ,m=5,S=NULL, verbose=F,
                             kapa,empri,coef.cat,L.cat,theta=NULL,
                             coeff.ridge,var.interest=NULL,
                             models.mice=vector("character", length = ncol(data)),...){
  # PURPOSE: calculate the pooled estimate
  # and the confidence interval for the quantities of interest
  # using a multiple imputation method
  
  #OUTPUT: the pooled estimate and the confidence interval for the quantities of interest
  
  #INPUTS:
  #data: an incomplete dataset
  #method: the multiple imputation methods used
  #formula.multinom: a string defining the analysis model
  #m: the numbers of imputed arays
  #S: the number of dimensions for MIMCA
  #verbose: if true, more details are printed
  #coeff.ridge: parameters for regularization with MIMCA
  #var.interest: index of the variables used in the analysis model
  #models.mice: the kind of modeling for FCS
  
  print(method)
  if(method=="MIMCA.w"){
    imp<-MIMCA(data,m=m,ncp=S,coeff.ridge=coeff.ridge)$X.imp
  }else if(method=="amelia"){
    imp<-amelia(data,m=m,p2s=if(verbose){1}else{0},empri=empri,noms=1:ncol(data))$imputations
  }else if(method=="LD"){
    return(LD.real(data=data,formula.multinom=formula.multinom,var.interest=var.interest))
  }else if(method=="DPMPM"){
    mdf<-missing_data.frame(data, subclass = "allcategorical")
    imp <- mi(mdf, n.chains = m,verbose=verbose)
    dfs <- mi::complete(imp, m = m) #extraction des jeux de donn?es imput?s
    imp<-lapply(dfs,'[',1:ncol(data))
    class(imp)<-"DPMPM"
  }else if(method=="da.cat"){
    datamod<-matrix(NA,nrow(data),ncol(data))
    for (i in 1:ncol(datamod)){
      datamod[,i]<-as.numeric(data[,i])
    }
    s<-prelim.cat(datamod)
    imp <- list()
    #initialisation
    rngseed(7817)
    if(is.null(theta)){theta.choose <- em.cat(s,showits=verbose)  }else{theta.choose<-theta}
    for (i in 1:m) {
      if(verbose){cat("Doing imputation ",i,"\n")}
      theta.choose <- da.cat(s,theta.choose,steps=L.cat,showits=verbose) 
      imp[[i]] <- imp.cat(s,theta.choose)
    } 
    imp<-lapply(imp,FUN=function(x,data){y<-x;colnames(y)<-colnames(data);return(y)},data=data)
    class(imp)<-"cat"
  }else if(method=="sample"){
    res.mice<- mice(data, m=m, print=verbose, ridge=kapa,method="sample",...)
    imp<-list()
    for (i in 1:m){imp[[i]]<-mice:::complete(res.mice,i)}
  }else{
    res.mice <- mice(data, m=m, print=verbose, ridge=kapa,method=models.mice,...)
    imp<-list()
    for (i in 1:m){imp[[i]]<-mice:::complete(res.mice,i)}
  }
  #pooling
  
  imp.cont<-lapply(imp,FUN=function(mat){
    xquali<-as.data.frame(mat)
    for(j in 1:ncol(mat)){xquali[,j]<-as.factor(mat[,j])}
    return(as.data.frame((xquali)))})
  Fit <- lapply(imp.cont, FUN=function(X,formula.multinom){
    multinom(as.formula(formula.multinom),data=as.data.frame(X),trace=F)},
                formula.multinom=formula.multinom)
  res<-summary(mice:::pool(as.mira(Fit)))[,c("est","lo 95","hi 95")]
  #permutation
  if(is.matrix(coef(Fit[[1]]))){
    res<-res[as.vector(matrix(t(matrix(1:nrow(res),nrow(coef(Fit[[1]])),ncol(coef(Fit[[1]])))),ncol=1)),]
  }
  return(res)
}

Simulate.real <- function(X,Y,nsim=10,p.na=0.2,m=5,n.ech,S,verbose=F,
                          methods=c("LD","sample","MIMCA.w"),empri=0,kapa=0,path.save=NULL,
                          L.cat=1000,theta=NULL,formula.multinom=NULL,coeff.ridge=1,
                          plot.sum=F,var.interest=NULL,
                          models.mice=NULL,Name="Jeu",...){
  #PURPOSE: perform simulation from a real dataset.
  
  #OUTPUTS: The outputs are the pooled estimates for each imputation method and each simulation,
  #as well as the summary of the study through the bias, the CI width and the coverage
  
  #INPUTS:
  # X: covariates
  # Y: response variable
  # nsim: number of simulation
  # p.na: percentage of missong values
  # m: number of imputed arrays
  # n.ech: size of the sample
  # S: number of dimensions for MIMCA
  # verbose: if true, more details about the simulation are printed
  # methods: the multiple imputation methods assessed
  # empri: regularization parameter for amelia
  # kapa: regularization parameter for mice
  # L.cat, theta: parameters for the loglinear model
  # coeff.ridge: regularization parameter for MIMCA
  # plot.sum: if true, graphics are ploted to sum up the current criteria 
  # var.interest: index of the variables used in the analysis model
  # models.mice: the kind of modelling for FCS
  # formula.multinom: regression formula that explains the response variable
  
  beta_true<-createdata.real(X,Y[,1,drop=F],n.ech=n.ech,formula.multinom=formula.multinom)$beta
  
  res_beta <- array(NA,dim=c(9, nsim, length(beta_true), 3))
  temp<-(if(coeff.ridge==1){"reg"}else if(coeff.ridge==0){"em"}else{paste("coeff.ridge=",coeff.ridge)})
  dimnames(res_beta) <- list(c("LD","Sample","Loglinear","DPMPM","Normal",paste("MIMCA",temp,S),"FCS-log","FCS-rf","Full data"),
                             as.character(1:nsim),
                             NULL,
                             c("est","lo 95","hi 95"))
  
  im <- c("LD","amelia","DPMPM","mice","da.cat","MIMCA.w")
  coef.cat<-c();for (i in 1:ncol(X)){for(j in (i+1):(ncol(X)+1)){coef.cat<-c(coef.cat,i,j,0)}}
  for(i in 1:nsim){
    print(i)
    set.seed(i*100)
    jeu <- createdata.real(X,Y[,1,drop=F],n.ech=n.ech,formula.multinom=formula.multinom)$X
    if(is.null(models.mice)){models.mice<-vector("character", length = ncol(jeu))}
    data.na<-jeu
    data.na <- prodNA(jeu,noNA=p.na)
    print(summary(data.na))
    if("LD"%in%methods){#LD
      temp<-test.impute.real(data.na, method=im[1],formula.multinom=formula.multinom,var.interest=var.interest)
      res_beta[1,i,,] <- temp
    }
    if("amelia"%in%methods){#Amelia
      temp<-test.impute.real(data.na, method=im[2],m=m,empri=empri,formula.multinom=formula.multinom,verbose=verbose)
      res_beta[5,i,,] <- temp
    }
    if ("DPMPM"%in%methods){#DPMPM
      temp<-test.impute.real(data.na, method=im[3],m=m,formula.multinom=formula.multinom,verbose=verbose)
      res_beta[4,i,,] <- temp
    }
    if("mice_log"%in%methods){#mice log
      temp<-try(test.impute.real(data.na, method=im[4],S=S,formula.multinom=formula.multinom,m=m,verbose=verbose,kapa=kapa,models.mice=models.mice))
      if(class(temp)!="try-error"){
        res_beta[7,i,,] <- temp
      }
    }
    if("mice_rf"%in%methods){#mice rf
      temp<-try(test.impute.real(data.na, method=im[4],S=S,formula.multinom=formula.multinom,m=m,verbose=verbose,kapa=kapa,models.mice="rf"))
      if(class(temp)!="try-error"){
        res_beta[8,i,,] <- temp
      }
    }
    if("da.cat"%in%methods){#logliner model
      temp<-test.impute.real(data.na, method=im[5],formula.multinom=formula.multinom,m=m,verbose=verbose,L.cat=L.cat,theta=theta)
      res_beta[4,i,,] <- temp
    }
    if("sample"%in%methods){#mice sample
      temp<-test.impute.real(data.na, method="sample",S=S,formula.multinom=formula.multinom,m=m,verbose=verbose,kapa=kapa)
      res_beta[2,i,,] <- temp
    }
    if("MIMCA.w"%in%methods){#MIMCA
      temp<-try(test.impute.real(data.na, method=im[6],S=S,formula.multinom=formula.multinom,m=m,verbose=verbose,coeff.ridge=coeff.ridge))
      if(class(temp)!="try-error"){
        res_beta[6,i,,] <- temp
      }
    }
    #Full Data
    temp<-test.impute.real(jeu, method="LD",formula.multinom=formula.multinom)
    res_beta[9,i,,] <- temp
    
    if(i>1){res.sum<-summarize.results(res_beta[,1:i,,],beta_true=beta_true);
            print(res.sum)
            if(!is.null(path.save)){
              res<-list(beta=res_beta,res.sum=res.sum)
              save(res,file=paste(path.save,"res_beta.Rdata",sep=""))
            }
            if(plot.sum){plot.summarize(res.sum=res.sum,beta_true=beta_true,nbsim=i,plot.cov=T,plot.ci=T,plot.bias=T,Name=Name)}
    }  
  }
  return(list(beta=res_beta,res.sum=res.sum))
}